@@ -1,5 +1,5 @@
 /* source: socat.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger 2001-2010 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the main source, including command line option parsing, general
@@ -1045,14 +1045,16 @@ int _socat(void) {
 	    xioshutdown(sock2, SHUT_WR);
 	    XIO_RDSTREAM(sock1)->eof = 2;
 	    XIO_RDSTREAM(sock1)->ignoreeof = false;
-	    if (socat_opts.lefttoright) {
-	       break;
-	    }
-	    closing = 1;
 	 }
       } else if (polling && XIO_RDSTREAM(sock1)->ignoreeof) {
 	 polling = 0;
       }
+      if (XIO_RDSTREAM(sock1)->eof >= 2) {
+	 if (socat_opts.lefttoright) {
+	    break;
+	 }
+	 closing = 1;
+      }
 
       if (bytes2 == 0 || XIO_RDSTREAM(sock2)->eof >= 2) {
 	 if (XIO_RDSTREAM(sock2)->ignoreeof &&
@@ -1066,14 +1068,16 @@ int _socat(void) {
 	    xioshutdown(sock1, SHUT_WR);
 	    XIO_RDSTREAM(sock2)->eof = 2;
 	    XIO_RDSTREAM(sock2)->ignoreeof = false;
-	    if (socat_opts.righttoleft) {
-	       break;
-	    }
-	    closing = 1;
 	 }
       } else if (polling && XIO_RDSTREAM(sock2)->ignoreeof) {
 	 polling = 0;
       }
+      if (XIO_RDSTREAM(sock2)->eof >= 2) {
+	 if (socat_opts.righttoleft) {
+	    break;
+	 }
+	 closing = 1;
+      }
    }
 
    /* close everything that's still open */
@@ -1293,7 +1297,8 @@ int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 	       }
 	    } else if (socat_opts.verbhex) {
 	       int i;
-	       /*! prefix? */
+	       /* print prefix */
+	       xioprintblockheader(stderr, bytes, righttoleft);
 	       for (i = 0; i < bytes; ++i) {
 		  fprintf(stderr, " %02x", (*buff)[i]);
 	       }
