@@ -1,5 +1,5 @@
-/* $Id: sycls.c,v 1.75 2007/03/06 21:04:12 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* source: sycls.c */
+/* Copyright Gerhard Rieger 2001-2009 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* explicit system call and C library trace function, for those who miss strace
@@ -18,6 +18,7 @@
 
 #include "error.h"
 #include "filan.h"
+#include "utils.h"
 #include "sysutils.h"
 #include "sycls.h"
 
@@ -332,7 +333,7 @@ int Gettimeofday(struct timeval *tv, struct timezone *tz) {
 
 int Mknod(const char *pathname, mode_t mode, dev_t dev) {
    int result, _errno;
-   Debug3("mknod(\"%s\", 0%o, %d)", pathname, mode, dev);
+   Debug3("mknod(\"%s\", 0%o, "F_dev")", pathname, mode, dev);
    result = mknod(pathname, mode, dev);
    _errno = errno;
    Debug1("mknod() -> %d", result);
@@ -353,11 +354,11 @@ int Mkfifo(const char *pathname, mode_t mode) {
 static void prtstat(const char *func, struct stat *buf, int result) {
    char txt[256], *t = txt;
 
-   t += sprintf(t, "%s(, {"F_st_dev","F_st_ino","F_mode","F_st_nlink","F_uid","F_gid,
+   t += sprintf(t, "%s(, {"F_dev","F_st_ino","F_mode","F_st_nlink","F_uid","F_gid,
 		func, buf->st_dev, buf->st_ino,
 		buf->st_mode, buf->st_nlink, buf->st_uid, buf->st_gid);
 #if HAVE_ST_RDEV
-   t += sprintf(t, ","F_st_dev, buf->st_rdev);
+   t += sprintf(t, ","F_dev, buf->st_rdev);
 #endif
    t += sprintf(t, ","F_st_size, buf->st_size);
 #if HAVE_ST_BLKSIZE
@@ -377,11 +378,11 @@ static void prtstat64(const char *func, struct stat64 *buf, int result) {
    if (result < 0) {
       sprintf(t, "%s(, {}) -> %d", func, result);
    } else {
-   t += sprintf(t, "%s(, {"F_st_dev","F_st64_ino","F_mode","F_st_nlink","F_uid","F_gid,
+   t += sprintf(t, "%s(, {"F_dev","F_st64_ino","F_mode","F_st_nlink","F_uid","F_gid,
 		func, buf->st_dev, buf->st_ino,
 		buf->st_mode, buf->st_nlink, buf->st_uid, buf->st_gid);
 #if HAVE_ST_RDEV
-   t += sprintf(t, ","F_st_dev, buf->st_rdev);
+   t += sprintf(t, ","F_dev, buf->st_rdev);
 #endif
    t += sprintf(t, ","F_st64_size, buf->st_size);
 #if HAVE_ST_BLKSIZE
@@ -593,6 +594,16 @@ int Ioctl(int d, int request, void *argp) {
    return retval;
 }
 
+int Ioctl_int(int d, int request, int arg) {
+   int retval, _errno;
+   Debug3("ioctl(%d, 0x%x, %d)", d, request, arg);
+   retval = ioctl(d, request, arg);
+   _errno = errno;
+   Debug1("ioctl() -> %d", retval);
+   errno = _errno;
+   return retval;
+}
+
 int Close(int fd) {
    int retval, _errno;
    Info1("close(%d)", fd);
@@ -677,9 +688,21 @@ int Chmod(const char *path, mode_t mode) {
 /* we only show the first struct pollfd; hope this is enough for most cases. */
 int Poll(struct pollfd *ufds, unsigned int nfds, int timeout) {
    int result;
-   Debug4("poll({%d, 0x%02hx, }, %u, %d)", ufds[0].fd, ufds[0].events, nfds, timeout);
+   if (nfds == 4) {
+      Debug10("poll({%d,0x%02hx,}{%d,0x%02hx,}{%d,0x%02hx,}{%d,0x%02hx,}, %u, %d)",
+	      ufds[0].fd, ufds[0].events, ufds[1].fd, ufds[1].events,
+	      ufds[2].fd, ufds[2].events, ufds[3].fd, ufds[3].events,
+	      nfds, timeout);
+   } else {
+      Debug4("poll({%d,0x%02hx,}, , %u, %d)", ufds[0].fd, ufds[0].events, nfds, timeout);
+   }
    result = poll(ufds, nfds, timeout);
-   Debug2("poll(, {,, 0x%02hx}) -> %d", ufds[0].revents, result);
+   if (nfds == 4) {
+      Debug5("poll(, {,,0x%02hx}{,,0x%02hx}{,,0x%02hx}{,,0x%02hx}) -> %d",
+	     ufds[0].revents, ufds[1].revents, ufds[2].revents, ufds[3].revents, result);
+   } else {
+      Debug2("poll(, {,,0x%02hx}) -> %d", ufds[0].revents, result);
+   }
    return result;
 }
 #endif /* HAVE_POLL */
@@ -1043,12 +1066,28 @@ int Recvfrom(int s, void *buf, size_t len, int flags, struct sockaddr *from,
 int Recvmsg(int s, struct msghdr *msgh, int flags) {
    int retval, _errno;
    char infobuff[256];
-   Debug3("recvmsg(%d, %p, %d)", s, msgh, flags);
+#if defined(HAVE_STRUCT_MSGHDR_MSGCONTROL) && defined(HAVE_STRUCT_MSGHDR_MSGCONTROLLEN) && defined(HAVE_STRUCT_MSGHDR_MSGFLAGS)
+   Debug10("recvmsg(%d, %p{%p,%u,%p,%u,%p,%u,%d}, %d)", s, msgh,
+	  msgh->msg_name, msgh->msg_namelen,  msgh->msg_iov,  msgh->msg_iovlen,
+	  msgh->msg_control,  msgh->msg_controllen,  msgh->msg_flags, flags);
+#else
+   Debug7("recvmsg(%d, %p{%p,%u,%p,%u}, %d)", s, msgh,
+	  msgh->msg_name, msgh->msg_namelen,  msgh->msg_iov,  msgh->msg_iovlen,
+	  flags);
+#endif
    retval = recvmsg(s, msgh, flags);
    _errno = errno;
-   Debug2("recvmsg(, {%s}, ) -> %d",
+#if defined(HAVE_STRUCT_MSGHDR_MSGCONTROLLEN)
+   Debug5("recvmsg(, {%s,%u,,%u,,%u,}, ) -> %d",
 	  msgh->msg_name?sockaddr_info(msgh->msg_name, msgh->msg_namelen, infobuff, sizeof(infobuff)):"NULL",
+	  msgh->msg_namelen, msgh->msg_iovlen, msgh->msg_controllen,
 	  retval);
+#else
+   Debug4("recvmsg(, {%s,%u,,%u,,}, ) -> %d",
+	  msgh->msg_name?sockaddr_info(msgh->msg_name, msgh->msg_namelen, infobuff, sizeof(infobuff)):"NULL",
+	  msgh->msg_namelen, msgh->msg_iovlen,
+	  retval);
+#endif
    errno = _errno;
    return retval;
 }
@@ -1233,6 +1272,7 @@ void *Realloc(void *ptr, size_t size) {
    return result;
 }
 
+#if _WITH_TERMIOS
 int Tcgetattr(int fd, struct termios *termios_p) {
    int i, result, _errno;
    char chars[5*NCCS], *cp = chars;
@@ -1252,7 +1292,9 @@ int Tcgetattr(int fd, struct termios *termios_p) {
    errno = _errno;
    return result;
 }
+#endif /* _WITH_TERMIOS */
 
+#if _WITH_TERMIOS
 int Tcsetattr(int fd, int optional_actions, struct termios *termios_p) {
    int i, result, _errno;
    char chars[5*NCCS], *cp = chars;
@@ -1270,6 +1312,7 @@ int Tcsetattr(int fd, int optional_actions, struct termios *termios_p) {
    errno = _errno;
    return result;
 }
+#endif /* _WITH_TERMIOS */
 
 char *Ttyname(int fd) {
    char *result;
@@ -1407,6 +1450,30 @@ int Mkstemp(char *template) {
    return result;
 }
 
+int Setenv(const char *name, const char *value, int overwrite) {
+   int result, _errno;
+   Debug3("setenv(\"%s\", \"%s\", %d)", name, value, overwrite);
+   result = setenv(name, value, overwrite);
+   _errno = errno;
+   Debug1("setenv() -> %d", result);
+   errno = _errno;
+   return result;
+}
+
+#if HAVE_UNSETENV
+/* on Linux it returns int but on FreeBSD void.
+   we do not expect many errors, so we take void which works on all systems. */
+void Unsetenv(const char *name) {
+   int _errno;
+   Debug1("unsetenv(\"%s\")", name);
+   unsetenv(name);
+   _errno = errno;
+   Debug("unsetenv() ->");
+   errno = _errno;
+   return;
+}
+#endif
+
 #if WITH_READLINE
 
 char *Readline(const char *prompt) {
