@@ -1,5 +1,5 @@
 /* source: xio-openssl.c */
-/* Copyright Gerhard Rieger 2002-2008 */
+/* Copyright Gerhard Rieger 2002-2011 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the implementation of the openssl addresses */
@@ -104,6 +104,9 @@ const struct optdesc opt_openssl_cafile      = { "openssl-cafile",     "cafile",
 const struct optdesc opt_openssl_capath      = { "openssl-capath",     "capath", OPT_OPENSSL_CAPATH,      GROUP_OPENSSL, PH_SPEC, TYPE_FILENAME, OFUNC_SPEC };
 const struct optdesc opt_openssl_egd         = { "openssl-egd",        "egd",    OPT_OPENSSL_EGD,         GROUP_OPENSSL, PH_SPEC, TYPE_FILENAME, OFUNC_SPEC };
 const struct optdesc opt_openssl_pseudo      = { "openssl-pseudo",     "pseudo", OPT_OPENSSL_PSEUDO,      GROUP_OPENSSL, PH_SPEC, TYPE_BOOL,     OFUNC_SPEC };
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
+const struct optdesc opt_openssl_compress    = { "openssl-compress",   "compress", OPT_OPENSSL_COMPRESS,  GROUP_OPENSSL, PH_SPEC, TYPE_STRING,   OFUNC_SPEC };
+#endif
 #if WITH_FIPS
 const struct optdesc opt_openssl_fips        = { "openssl-fips",       "fips",   OPT_OPENSSL_FIPS,        GROUP_OPENSSL, PH_SPEC, TYPE_BOOL,     OFUNC_SPEC };
 #endif
@@ -134,6 +137,24 @@ int xio_reset_fips_mode(void) {
 #define xio_reset_fips_mode() 0
 #endif
 
+static void openssl_conn_loginfo(SSL *ssl) {
+   Notice1("SSL connection using %s", SSL_get_cipher(ssl));
+
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
+   {
+      const COMP_METHOD *comp, *expansion;
+
+      comp = sycSSL_get_current_compression(ssl);
+      expansion = sycSSL_get_current_expansion(ssl);
+
+      Notice1("SSL connection compression \"%s\"",
+              comp?sycSSL_COMP_get_name(comp):"none");
+      Notice1("SSL connection expansion \"%s\"",
+              expansion?sycSSL_COMP_get_name(expansion):"none");
+   }
+#endif
+}
+
 /* the open function for OpenSSL client */
 static int
    xioopen_openssl_connect(int argc,
@@ -200,7 +221,7 @@ static int
 			     xfd->para.socket.ip.res_opts[1],
 			     xfd->para.socket.ip.res_opts[0],
 			     them, &themlen, us, &uslen,
-			     &needbind, &lowport, &socktype);
+			     &needbind, &lowport, socktype);
    if (result != STAT_OK)  return STAT_NORETRY;
 
    if (xioopts.logopt == 'm') {
@@ -275,32 +296,23 @@ static int
 #if WITH_RETRY
       if (dofork) {
 	 pid_t pid;
-	 while ((pid = Fork()) < 0) {
-	    int level = E_ERROR;
-	    if (xfd->forever || xfd->retry) {
-	       level = E_WARN;
-	    }
-	    Msg1(level, "fork(): %s", strerror(errno));
-	    if (xfd->forever || xfd->retry) {
-	       Nanosleep(&xfd->intervall, NULL);
-	       --xfd->retry;
-	       continue;
+	 int level = E_ERROR;
+	 if (xfd->forever || xfd->retry) {
+	    level = E_WARN;
+	 }
+	 while ((pid = xio_fork(false, level)) < 0) {
+	    if (xfd->forever || --xfd->retry) {
+	       Nanosleep(&xfd->intervall, NULL); continue;
 	    }
 	    return STAT_RETRYLATER;
 	 }
-	 if (pid == 0) {	/* child process */
-	    Info1("just born: OpenSSL client process "F_pid, Getpid());
 
-	    /* drop parents locks, reset FIPS... */
-	    if (xio_forked_inchild() != 0) {
-	       Exit(1);
-	    }
-	    xfd->forever = false;
-	    xfd->retry = 0;
+	 if (pid == 0) {	/* child process */
+	    xfd->forever = false;  xfd->retry = 0;
 	    break;
 	 }
+
 	 /* parent process */
-	 Notice1("forked off child process "F_pid, pid);
 	 Close(xfd->fd);
 	 sycSSL_free(xfd->para.openssl.ssl);
 	 xfd->para.openssl.ssl = NULL;
@@ -313,7 +325,7 @@ static int
       break;
    } while (true);	/* drop out on success */
 
-   Notice1("SSL connection using %s", SSL_get_cipher(xfd->para.openssl.ssl));
+   openssl_conn_loginfo(xfd->para.openssl.ssl);
 
    /* fill in the fd structure */
    return STAT_OK;
@@ -440,7 +452,7 @@ static int
    if (_xioopen_ipapp_listen_prepare(opts, &opts0, portname, &pf, ipproto,
 				     xfd->para.socket.ip.res_opts[1],
 				     xfd->para.socket.ip.res_opts[0],
-				     us, &uslen, &socktype)
+				     us, &uslen, socktype)
        != STAT_OK) {
       return STAT_NORETRY;
    }
@@ -510,8 +522,7 @@ static int
 	 return result;
       }
 
-      Notice1("SSL connection using %s",
-	      SSL_get_cipher(xfd->para.openssl.ssl));
+      openssl_conn_loginfo(xfd->para.openssl.ssl);
       break;
 
    }	/* drop out on success */
@@ -615,6 +626,59 @@ int _xioopen_openssl_listen(struct single *xfd,
 #endif /* WITH_LISTEN */
 
 
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
+/* In OpenSSL 0.9.7 compression methods could be added using
+ * SSL_COMP_add_compression_method(3), but the implemntation is not compatible
+ * with the standard (RFC3749).
+ */
+static int openssl_setup_compression(SSL_CTX *ctx, char *method)
+{
+   STACK_OF(SSL_COMP)* comp_methods;
+
+   assert(method);
+
+   /* Getting the stack of compression methods has the intended side-effect of
+    * initializing the SSL library's compression part.
+    */
+   comp_methods = SSL_COMP_get_compression_methods();
+   if (!comp_methods) {
+      Info("OpenSSL built without compression support");
+      return STAT_OK;
+   }
+
+   if (strcasecmp(method, "auto") == 0) {
+      Info("Using default OpenSSL compression");
+      return STAT_OK;
+   }
+
+   if (strcasecmp(method, "none") == 0) {
+      /* Disable compression */
+#ifdef SSL_OP_NO_COMPRESSION
+      Info("Disabling OpenSSL compression");
+      SSL_CTX_set_options(ctx, SSL_OP_NO_COMPRESSION);
+#else
+      /* SSL_OP_NO_COMPRESSION was only introduced in OpenSSL 0.9.9 (released
+       * as 1.0.0). Removing all compression methods is a work-around for
+       * earlier versions of OpenSSL, but it affects all SSL connections.
+       */
+      Info("Disabling OpenSSL compression globally");
+      sk_SSL_COMP_zero(comp_methods);
+#endif
+      return STAT_OK;
+   }
+
+   /* zlib compression in OpenSSL before version 0.9.8e-beta1 uses the libc's
+    * default malloc/free instead of the ones passed to OpenSSL. Should socat
+    * ever use custom malloc/free functions for OpenSSL, this must be taken
+    * into consideration. See OpenSSL bug #1468.
+    */
+
+   Error1("openssl-compress=\"%s\": unknown compression method", method);
+   return STAT_NORETRY;
+}
+#endif
+
+
 int
    _xioopen_openssl_prepare(struct opt *opts,
 			    struct single *xfd,/* a xio file descriptor
@@ -626,7 +690,7 @@ int
 			    SSL_CTX **ctx)
 {
    bool opt_fips = false;
-   SSL_METHOD *method;
+   const SSL_METHOD *method;
    char *me_str = NULL;	/* method string */
    char *ci_str = NULL;	/* cipher string */
    char *opt_key  = NULL;	/* file name of client private key */
@@ -634,6 +698,9 @@ int
    char *opt_cafile = NULL;	/* certificate authority file */
    char *opt_capath = NULL;	/* certificate authority directory */
    char *opt_egd = NULL;	/* entropy gathering daemon socket path */
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
+   char *opt_compress = NULL;	/* compression method */
+#endif
    bool opt_pseudo = false;	/* use pseudo entropy if nothing else */
    unsigned long err;
    int result;
@@ -651,6 +718,9 @@ int
    retropt_string(opts, OPT_OPENSSL_DHPARAM, &opt_dhparam);
    retropt_string(opts, OPT_OPENSSL_EGD, &opt_egd);
    retropt_bool(opts,OPT_OPENSSL_PSEUDO, &opt_pseudo);
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
+   retropt_string(opts, OPT_OPENSSL_COMPRESS, &opt_compress);
+#endif
 
 #if WITH_FIPS
    if (opt_fips) {
@@ -677,8 +747,14 @@ int
    if (!server) {
       if (me_str != 0) {
 	 if (!strcasecmp(me_str, "SSLv2") || !strcasecmp(me_str, "SSL2")) {
+#if HAVE_SSLv2_client_method
 	    method = sycSSLv2_client_method();
-	 } else if (!strcasecmp(me_str, "SSLv3") || !strcasecmp(me_str, "SSL3")) {
+#else
+	    Error1("OpenSSL method \"%s\" not provided by library", me_str);
+	    method = sycSSLv23_server_method();
+#endif
+	 } else
+	 if (!strcasecmp(me_str, "SSLv3") || !strcasecmp(me_str, "SSL3")) {
 	    method = sycSSLv3_client_method();
 	 } else if (!strcasecmp(me_str, "SSLv23") || !strcasecmp(me_str, "SSL23") ||
 		    !strcasecmp(me_str, "SSL")) {
@@ -688,16 +764,22 @@ int
 	    method = sycTLSv1_client_method();
 	 } else {
 	    Error1("openssl-method=\"%s\": unknown method", me_str);
-	    method = sycSSLv23_client_method()/*!*/;
+	    method = sycSSLv23_client_method();
 	 }
       } else {
-	 method = sycSSLv23_client_method()/*!*/;
+	 method = sycSSLv23_client_method();
       }
    } else /* server */ {
       if (me_str != 0) {
 	 if (!strcasecmp(me_str, "SSLv2") || !strcasecmp(me_str, "SSL2")) {
+#if HAVE_SSLv2_server_method
 	    method = sycSSLv2_server_method();
-	 } else if (!strcasecmp(me_str, "SSLv3") || !strcasecmp(me_str, "SSL3")) {
+#else
+	    Error1("OpenSSL method \"%s\" not provided by library", me_str);
+	    method = sycSSLv23_server_method();
+#endif
+	 } else
+	 if (!strcasecmp(me_str, "SSLv3") || !strcasecmp(me_str, "SSL3")) {
 	    method = sycSSLv3_server_method();
 	 } else if (!strcasecmp(me_str, "SSLv23") || !strcasecmp(me_str, "SSL23") ||
 		    !strcasecmp(me_str, "SSL")) {
@@ -707,10 +789,10 @@ int
 	    method = sycTLSv1_server_method();
 	 } else {
 	    Error1("openssl-method=\"%s\": unknown method", me_str);
-	    method = sycSSLv23_server_method()/*!*/;
+	    method = sycSSLv23_server_method();
 	 }
       } else {
-	 method = sycSSLv23_server_method()/*!*/;
+	 method = sycSSLv23_server_method();
       }
    }
 
@@ -748,6 +830,60 @@ int
       return STAT_RETRYLATER;
    }
 
+   {
+      static unsigned char dh512_p[] = {
+	 0xDA,0x58,0x3C,0x16,0xD9,0x85,0x22,0x89,0xD0,0xE4,0xAF,0x75,
+	 0x6F,0x4C,0xCA,0x92,0xDD,0x4B,0xE5,0x33,0xB8,0x04,0xFB,0x0F,
+	 0xED,0x94,0xEF,0x9C,0x8A,0x44,0x03,0xED,0x57,0x46,0x50,0xD3,
+	 0x69,0x99,0xDB,0x29,0xD7,0x76,0x27,0x6B,0xA2,0xD3,0xD4,0x12,
+	 0xE2,0x18,0xF4,0xDD,0x1E,0x08,0x4C,0xF6,0xD8,0x00,0x3E,0x7C,
+	 0x47,0x74,0xE8,0x33,
+      };
+      static unsigned char dh512_g[] = {
+	 0x02,
+      };
+      DH *dh;
+      unsigned long err;
+
+      if ((dh = DH_new()) == NULL) {
+	 while (err = ERR_get_error()) {
+	    Warn1("DH_new(): %s",
+		   ERR_error_string(err, NULL));
+	 }
+	 Error("DH_new() failed");
+      } else {
+	 dh->p = BN_bin2bn(dh512_p, sizeof(dh512_p), NULL);
+	 dh->g = BN_bin2bn(dh512_g, sizeof(dh512_g), NULL);
+	 if ((dh->p == NULL) || (dh->g == NULL)) {
+	    while (err = ERR_get_error()) {
+	       Warn1("BN_bin2bn(): %s",
+		     ERR_error_string(err, NULL));
+	    }
+	    Error("BN_bin2bn() failed");
+	 } else {
+	    if (SSL_CTX_set_tmp_dh(*ctx, dh) <= 0) {
+	       while (err = ERR_get_error()) {
+		  Warn3("SSL_CTX_set_tmp_dh(%p, %p): %s", *ctx, dh,
+			ERR_error_string(err, NULL));
+	       }
+	       Error2("SSL_CTX_set_tmp_dh(%p, %p) failed", *ctx, dh);
+	    }
+	    /*! OPENSSL_free(dh->p,g)? doc does not tell so */
+	 }
+	 DH_free(dh);
+      }
+   }
+
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
+   if (opt_compress) {
+      int result;
+      result = openssl_setup_compression(*ctx, opt_compress);
+      if (result != STAT_OK) {
+	return result;
+      }
+   }
+#endif
+
    if (opt_cafile != NULL || opt_capath != NULL) {
       if (sycSSL_CTX_load_verify_locations(*ctx, opt_cafile, opt_capath) != 1) {
 	 int result;
@@ -1063,8 +1199,9 @@ ssize_t xioread_openssl(struct single *pipe, void *buff, size_t bufsiz) {
       case SSL_ERROR_WANT_WRITE:
       case SSL_ERROR_WANT_CONNECT:
       case SSL_ERROR_WANT_X509_LOOKUP:
-	 Error("nonblocking operation did not complete");
-	 break;	/*!*/
+	 Info("nonblocking operation did not complete");
+	 errno = EAGAIN;
+	 return -1;
       case SSL_ERROR_SYSCALL:
 	 if (ERR_peek_error() == 0) {
 	    if (ret == 0) {
