@@ -1,5 +1,5 @@
-/* $Id: xio-proxy.h,v 1.4 2004/06/06 17:33:22 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2004 */
+/* source: xio-proxy.h */
+/* Copyright Gerhard Rieger 2002-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_proxy_h_included
@@ -11,7 +11,7 @@ struct proxyvars {
    bool doresolve;
    char *authstring;
    char *targetaddr;	/* name/address of host, in malloced string */
-   byte2_t targetport;
+   uint16_t targetport;
 } ;
 
 extern const struct optdesc opt_proxyport;
