@@ -1,5 +1,5 @@
-/* $Id: xio-socks.c,v 1.33 2006/12/28 14:06:37 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* source: xio-socks.c */
+/* Copyright Gerhard Rieger 2001-2011 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of socks4 type */
@@ -88,7 +88,7 @@ static int xioopen_socks4_connect(int argc, const char *argv[], struct opt *opts
 			     xfd->para.socket.ip.res_opts[1],
 			     xfd->para.socket.ip.res_opts[0],
 			     them, &themlen, us, &uslen,
-			     &needbind, &lowport, &socktype);
+			     &needbind, &lowport, socktype);
 
    Notice5("opening connection to %s:%u via socks4 server %s:%s as user \"%s\"",
 	   targetname,
@@ -163,33 +163,32 @@ static int xioopen_socks4_connect(int argc, const char *argv[], struct opt *opts
 	 return result;
       }
 
+      if (dofork) {
+	 xiosetchilddied();	/* set SIGCHLD handler */
+      }
+
 #if WITH_RETRY
       if (dofork) {
 	 pid_t pid;
-	 while ((pid = Fork()) < 0) {
-	    int level = E_ERROR;
-	    if (xfd->forever || xfd->retry) {
-	       level = E_WARN;
-	    }
-	    Msg1(level, "fork(): %s", strerror(errno));
-	    if (xfd->forever || xfd->retry--) {
+	 int level = E_ERROR;
+	 if (xfd->forever || xfd->retry) {
+	    level = E_WARN;	/* most users won't expect a problem here,
+				   so Notice is too weak */
+	 }
+	 while ((pid = xio_fork(false, level)) < 0) {
+	    if (xfd->forever || --xfd->retry) {
 	       Nanosleep(&xfd->intervall, NULL);
 	       continue;
 	    }
 	    return STAT_RETRYLATER;
 	 }
-	 if (pid == 0) {	/* child process */
-	    Info1("just born: socks client process "F_pid, Getpid());
 
-	    /* drop parents locks, reset FIPS... */
-	    if (xio_forked_inchild() != 0) {
-	       Exit(1);
-	    }
+	 if (pid == 0) {	/* child process */
 	    xfd->forever = false;  xfd->retry = 0;
 	    break;
 	 }
+
 	 /* parent process */
-	 Notice1("forked off child process "F_pid, pid);
 	 Close(xfd->fd);
 	 Nanosleep(&xfd->intervall, NULL);
 	 dropopts(opts, PH_ALL); opts = copyopts(opts0, GROUP_ALL);
@@ -212,7 +211,8 @@ int _xioopen_socks4_prepare(const char *targetport, struct opt *opts, char **soc
    /* generate socks header - points to final target */
    sockhead->version = 4;
    sockhead->action  = 1;
-   sockhead->port    = parseport(targetport, IPPROTO_TCP);
+   sockhead->port    = parseport(targetport, IPPROTO_TCP);	/* network byte
+								   order */
 
    if (retropt_string(opts, OPT_SOCKSPORT, socksport) < 0) {
       if ((se = getservbyname("socks", "tcp")) != NULL) {
@@ -310,7 +310,7 @@ int _xioopen_socks4_connect(struct single *xfd,
    }
    Info11("sending socks4%s request VN=%d DC=%d DSTPORT=%d DSTIP=%d.%d.%d.%d USERID=%s%s%s",
 	  destdomname?"a":"",
-	  sockhead->version, sockhead->action, sockhead->port,
+	  sockhead->version, sockhead->action, ntohs(sockhead->port),
 	  ((unsigned char *)&sockhead->dest)[0],
 	  ((unsigned char *)&sockhead->dest)[1],
 	  ((unsigned char *)&sockhead->dest)[2],
@@ -328,10 +328,7 @@ int _xioopen_socks4_connect(struct single *xfd,
       }
    }
 #endif /* WITH_MSGLEVEL <= E_DEBUG */
-   do {
-      result = Write(xfd->fd, sockhead, headlen);
-   } while (result < 0 && errno == EINTR);
-   if (result < 0) {
+   if (writefull(xfd->fd, sockhead, headlen) < 0) {
       Msg4(level, "write(%d, %p, "F_Zu"): %s",
 	   xfd->fd, sockhead, headlen, strerror(errno));
       if (Close(xfd->fd) < 0) {
