@@ -1,5 +1,5 @@
 /* source: xio.h */
-/* Copyright Gerhard Rieger 2001-2009 */
+/* Copyright Gerhard Rieger 2001-2011 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_h_included
@@ -158,7 +158,6 @@ typedef struct single {
       END_NONE,		/* no action */
       END_CLOSE,	/* close() */
       END_SHUTDOWN,	/* shutdown() */
-      END_UNLINK,	/* unlink() */
       END_KILL,		/* has subprocess */
       END_CLOSE_KILL,	/* first close fd, then kill subprocess */
       END_SHUTDOWN_KILL	/* first shutdown fd, then kill subprocess */
@@ -171,7 +170,6 @@ typedef struct single {
    bool ttyvalid;		/* the following struct is valid */
    struct termios savetty;	/* save orig tty settings for later restore */
 #endif /* WITH_TERMIOS */
-   const char *name;		/* only with END_UNLINK */
    int (*sigchild)(struct single *);	/* callback after sigchild */
    pid_t ppid;			/* parent pid, only if we send it signals */
    int escape;			/* escape character; -1 for no escape */
@@ -380,6 +378,8 @@ struct opt {
 extern const char *PIPESEP;
 extern xiofile_t *sock[XIO_MAXSOCK];
 
+extern int num_child;
+
 /* return values of xioopensingle */
 #define STAT_OK		0
 #define STAT_WARNING	1
@@ -404,10 +404,12 @@ extern int xioopenhelp(FILE *of, int level);
 
 /* must be outside function for use by childdied handler */
 extern xiofile_t *sock1, *sock2;
-extern pid_t diedunknown1;	/* child died before it is registered */
-extern pid_t diedunknown2;
-extern pid_t diedunknown3;
-extern pid_t diedunknown4;
+#define NUMUNKNOWN 4
+extern pid_t diedunknown[NUMUNKNOWN];	/* child died before it is registered */
+#define diedunknown1 (diedunknown[0])
+#define diedunknown2 (diedunknown[1])
+#define diedunknown3 (diedunknown[2])
+#define diedunknown4 (diedunknown[3])
 
 extern int xiosetsigchild(xiofile_t *xfd, int (*callback)(struct single *));
 extern int xiosetchilddied(void);
