@@ -1,5 +1,5 @@
 /* source: xioopts.h */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger 2001-2009 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xioopts_h_included
@@ -10,6 +10,7 @@
 #define ODESC_ERROR ODESC_DONE	/* maybe later */
 
 #define XIO_OFFSETOF(x) ((size_t)&((xiosingle_t *)0)->x)
+#define XIO_SIZEOF(x) (sizeof(((struct single *)0)->x))
 
 /* atomic structure for use in the option search table; keep compatible with
    struct wordent! */
@@ -20,7 +21,7 @@ struct optname {
 
 /* keep consistent with xiohelp.c:optiontypenames[] ! */
 enum e_types {
-   TYPE_CONST,		/* keyword means a fix value */
+   TYPE_CONST,		/* keyword means a fix value - implies int type */
    TYPE_BIN,		/* raw binary data, length determined by data */
    TYPE_BOOL,		/* value is 0 or 1 (no-value is interpreted as 1) */
    TYPE_BYTE,		/* unsigned char */
@@ -431,6 +432,7 @@ enum e_optcode {
    OPT_IXON,		/* termios.c_iflag */
    OPT_LOCKFILE,
    OPT_LOWPORT,
+   OPT_MAX_CHILDREN,
 #ifdef NLDLY
 #  ifdef NL0
    OPT_NL0,		/* termios.c_oflag */
@@ -444,6 +446,7 @@ enum e_optcode {
    OPT_NOFLSH,		/* termios.c_lflag */
    OPT_NOFORK,		/* exec, system */
    OPT_NOPROMPT,	/* readline */
+   OPT_NULL_EOF,	/* receiving empty packet triggers EOF */
 #ifdef OCRNL
    OPT_OCRNL,		/* termios.c_oflag */
 #endif
@@ -470,6 +473,9 @@ enum e_optcode {
    OPT_OPENSSL_CAPATH,
    OPT_OPENSSL_CERTIFICATE,
    OPT_OPENSSL_CIPHERLIST,
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
+   OPT_OPENSSL_COMPRESS,
+#endif
    OPT_OPENSSL_DHPARAM,
    OPT_OPENSSL_EGD,
    OPT_OPENSSL_FIPS,
@@ -590,6 +596,10 @@ enum e_optcode {
    OPT_SETSOCKOPT_STRING,
    OPT_SETUID,
    OPT_SETUID_EARLY,
+   OPT_SHUT_CLOSE,
+   OPT_SHUT_DOWN,
+   OPT_SHUT_NONE,
+   OPT_SHUT_NULL,	/* send 0 bytes on shutdown */
    OPT_SIGHUP,
    OPT_SIGINT,
    OPT_SIGQUIT,
@@ -695,8 +705,11 @@ enum e_optcode {
 #  define ENABLE_OPTCODE
 #  include "xio-streams.h"
 #  undef ENABLE_OPTCODE
+   OPT_SUBSTUSER_EARLY,
    OPT_SUBSTUSER,
+#if defined(HAVE_SETGRENT) && defined(HAVE_GETGRENT) && defined(HAVE_ENDGRENT)
    OPT_SUBSTUSER_DELAYED,
+#endif
    OPT_SYMBOLIC_LINK,	/* with pty */
 #ifdef TABDLY
 #  ifdef TAB0
@@ -929,4 +942,52 @@ extern int _groupbits(mode_t mode);
 extern int dropopts(struct opt *opts, unsigned int phase);
 extern int dropopts2(struct opt *opts, unsigned int from, unsigned int to);
 
+#if HAVE_BASIC_UID_T==1
+#  define retropt_uid(o,c,r) retropt_short(o,c,r)
+#elif HAVE_BASIC_UID_T==2
+#  define retropt_uid(o,c,r) retropt_ushort(o,c,r)
+#elif HAVE_BASIC_UID_T==3
+#  define retropt_uid(o,c,r) retropt_int(o,c,r)
+#elif HAVE_BASIC_UID_T==4
+#  define retropt_uid(o,c,r) retropt_uint(o,c,r)
+#elif HAVE_BASIC_UID_T==5
+#  define retropt_uid(o,c,r) retropt_long(o,c,r)
+#elif HAVE_BASIC_UID_T==6
+#  define retropt_uid(o,c,r) retropt_ulong(o,c,r)
+#else
+#  error "HAVE_BASIC_UID_T is out of range: " HAVE_BASIC_UID_T
+#endif
+
+#if HAVE_BASIC_GID_T==1
+#  define retropt_gid(o,c,r) retropt_short(o,c,r)
+#elif HAVE_BASIC_GID_T==2
+#  define retropt_gid(o,c,r) retropt_ushort(o,c,r)
+#elif HAVE_BASIC_GID_T==3
+#  define retropt_gid(o,c,r) retropt_int(o,c,r)
+#elif HAVE_BASIC_GID_T==4
+#  define retropt_gid(o,c,r) retropt_uint(o,c,r)
+#elif HAVE_BASIC_GID_T==5
+#  define retropt_gid(o,c,r) retropt_long(o,c,r)
+#elif HAVE_BASIC_GID_T==6
+#  define retropt_gid(o,c,r) retropt_ulong(o,c,r)
+#else
+#  error "HAVE_BASIC_GID_T is out of range: " HAVE_BASIC_GID_T
+#endif
+
+#if HAVE_BASIC_MODE_T==1
+#  define retropt_mode(o,c,r) retropt_short(o,c,r)
+#elif HAVE_BASIC_MODE_T==2
+#  define retropt_mode(o,c,r) retropt_ushort(o,c,r)
+#elif HAVE_BASIC_MODE_T==3
+#  define retropt_mode(o,c,r) retropt_int(o,c,r)
+#elif HAVE_BASIC_MODE_T==4
+#  define retropt_mode(o,c,r) retropt_uint(o,c,r)
+#elif HAVE_BASIC_MODE_T==5
+#  define retropt_mode(o,c,r) retropt_long(o,c,r)
+#elif HAVE_BASIC_MODE_T==6
+#  define retropt_mode(o,c,r) retropt_ulong(o,c,r)
+#else
+#  error "HAVE_BASIC_MODE_T is out of range: " HAVE_BASIC_MODE_T
+#endif
+
 #endif /* !defined(__xioopts_h_included) */
